@@ -1,5 +1,6 @@
 # Copyright (c) 2013 Paul Tagliamonte <paultag@debian.org>
 # Copyright (c) 2013 Julien Danjou <julien@danjou.info>
+# Copyright (c) 2013 Berker Peksag <berker.peksag@gmail.com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -23,9 +24,24 @@ try:
     import __builtin__ as builtins
 except ImportError:
     import builtins  # NOQA
+try:
+    from py_compile import MAGIC, wr_long
+except ImportError:
+    # py_compile.MAGIC removed and imp.get_magic() deprecated in Python 3.4
+    from importlib.util import MAGIC_NUMBER as MAGIC  # NOQA
+
+    def wr_long(f, x):
+        """Internal; write a 32-bit int to a file in little-endian order."""
+        f.write(bytes([x & 0xff,
+                       (x >> 8) & 0xff,
+                       (x >> 16) & 0xff,
+                       (x >> 24) & 0xff]))
 import sys
 
+PY27 = sys.version_info >= (2, 7)
 PY3 = sys.version_info[0] >= 3
+PY33 = sys.version_info >= (3, 3)
+PY34 = sys.version_info >= (3, 4)
 
 if PY3:
     str_type = str
